#if defined(_WIN32)

#include <windows.h>
#include <stdio.h>
#include <ctype.h>   /* for isprint */

#else
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#endif

#define int8_t		char 
#define uint8_t		unsigned char 
#define int16_t		short 
#define uint16_t	unsigned short 

#include "serial.h"

char * progname = "adctest";    /* name of program, for messages */
int verbose;             	/* verbosity level (-v, -vv, ...) */

union filedescriptor ser_fd;

extern void ser_open(char * port, long baud, union filedescriptor *fd);
extern int ser_setspeed(union filedescriptor *fd, long baud);
extern void ser_close(union filedescriptor *fd);

extern int ser_send(union filedescriptor *fd, unsigned char * buf, size_t buflen
);
extern int ser_recv(union filedescriptor *fd, unsigned char * buf, size_t buflen
);
extern int ser_drain(union filedescriptor *fd, int display);

extern int ser_set_dtr_rts(union filedescriptor *fd, int is_on);
#define ADC_CHAN_VCC            0x6 //
#define ADC_CHAN_TEMP           0x7 // internal #4 (TEMP)

#define ADC_CHAN_AIN31          0x8
#define ADC_CHAN_AIN32          0x9
#define ADC_CHAN_AIN41          0xa
#define ADC_CHAN_AIN42          0xb
#define ADC_CHAN_AIN21          0xc

#define ADC_CHAN_DELTA          0x08
#define ADC_CHAN_MASK           0x0f

/* flags for DELTA mode */
#define ADC_RANGE_AUTO          0x00 // Auto Range
#define ADC_RANGE_V512          0x10 // Reference VCC
#define ADC_RANGE_V256          0x20 // Reference 2.56V
#define DEV_ADC         1

double adc_read(union filedescriptor *fd, int opt)
{
	uint16_t result;
	char buf[256];
	double r;
	buf[0] = 's';
	buf[1] = DEV_ADC;
	buf[2] = 1; // bytes
	buf[3] = opt;
	ser_send(fd, (uint8_t *)buf, 4);
	ser_recv(fd, (uint8_t *)buf, 1); // return val (bytes)
	if (buf[0] != 1) {
		fprintf(stderr,"ADC send error\n");
		exit(1);
	}
	buf[0] = 'r';
	buf[1] = DEV_ADC;
	buf[2] = 2;
	ser_send(fd, (uint8_t *)buf, 3);
	ser_recv(fd, (uint8_t *)buf, 3);
	if (buf[0] != 2) {
		fprintf(stderr,"ADC recv error\n");
		exit(1);
	}
	result = buf[1] + buf[2]* 256;
	r = result;
	r = r * 10.0 / 128.0 / 1000.0; // 10/128 mV
	return r;
}

int main(int argc, char *argv[])
{
#ifdef _WIN32
	char *port = "COM13";
#else
	char *port = "/dev/ttyACM0";
#endif
	union filedescriptor *fd = &ser_fd;
	int i;
	double v;
	char buf[256];

	if ((argc > 2) && !strcmp(argv[1], "-P")) port = argv[2];

printf("ser_open port = %s baud = 115200 \n",port);
fflush(stdout);
	ser_open(port, 115200, fd);
	ser_set_dtr_rts(fd, 0);
	usleep(50*1000);
	ser_drain(fd, 0);
	ser_send(fd, (uint8_t *)"S", 1);
	ser_recv(fd, (uint8_t *)buf, 7);
	buf[7] = 0;
	if (strncmp(buf, "SERJTAG", 7)) {
		fprintf(stderr,"device is not serjtag\n");
		exit(1);
	}
	ser_send(fd, (uint8_t *)"V", 1);
	ser_recv(fd, (uint8_t *)buf, 2);
	buf[2] = 0;
	if (strncmp(buf, "21", 2) && strncmp(buf, "23", 2)) {
		fprintf(stderr,"device is not supprt ADC\n");
		exit(1);
	}
	ser_send(fd, (uint8_t *)"e", 1); // enter extmode 1
	ser_recv(fd, (uint8_t *)buf, 1);
	if (strncmp((char *)buf, "Y", 1)) {
		fprintf(stderr,"can't enter extmode\n");
		exit(1);
	}
	for (i=0; i<3; i++) {
		v = adc_read(fd, ADC_RANGE_V256 | ADC_CHAN_VCC);
		printf("ADC_RANGE_V256 | ADC_CHAN_VCC : %7.5f\n",v);
	}
	for (i=0; i<3; i++) {
		v = adc_read(fd, ADC_RANGE_V512 | ADC_CHAN_VCC);
		printf("ADC_RANGE_V512 | ADC_CHAN_VCC : %7.5f\n",v);
	}
	for (i=0; i<3; i++) {
		v = adc_read(fd, ADC_RANGE_V256 | ADC_CHAN_TEMP);
		printf("ADC_RANGE_V256 | ADC_CHAN_TEMP : %7.5f\n",v);
	}
	ser_close(fd);
	return 0;
}
