#define TX_SIZE         64  /* USB recieve buffer size (must be 2^n, 16<= ~ <=256 ) */
#define RX_SIZE         32  /* USB transmit buffer size (must be 2^n ) <=256 */

#define TGT_PORT	PORTB
#define TGT_DDR		DDRB
#define TGT_PIN		PINB

#define TGT_MISO	PB3
#define TGT_SCK		PB1
#define TGT_MOSI	PB2
#define TGT_RESET	PB0

#define __CDC162__

#define JTAG_EXTENSION
//#define ADC_EXTENSION
#define I2C_EXTENSION
//#define I2C_NOCHECK_START
#define NO_DEV_ECHO


#define MONITOR_SET_CONTROL	0
#define MONITOR_SEND_BREAK	0

#define MIN_LIGHTING_TIME 	5	// x 10ms
static inline void led_tx_on() {
	bit_clear(PORTD, PD5);
}
static inline void led_tx_off() {
	bit_set(PORTD, PD5)
}
static inline void led_rx_on() {
	bit_clear(PORTD, PD4);
}
static inline void led_rx_off() {
	bit_set(PORTD, PD4)
}

static inline void led_init() {
	bit_set(DDRD, PD4);
	bit_set(DDRD, PD5);
}

extern void serjtag_poll(void);
extern void serjtag_open(void);
extern void serjtag_close(void);

extern long Baud;

static inline void serjtag_busy_led_on() {
	led_tx_off();
}
static inline void serjtag_busy_led_off() {
	led_tx_on();
}

static inline void clkout_start() {
	// 2M Hz timer = 1M Hz clockout
	DDRC |= (1<< PC5);
	OCR1A =  (F_CPU/1000/2000 - 1) & 0xffff;
	TCCR1C = 0;
	TCCR1A = (2<<COM1B0) | (0<<WGM10); // OC1B toggle
	TCCR1B = (1<<WGM12)|(1<<CS10);  // CTC  1/1
}

static inline void clkout_stop() {
	DDRC &= ~(1<< PC5);
	TCCR1B = 0;
	TCCR1A = 0;
}

static inline void dtr_rts_init() {
	bit_set(DDRD, PD7);
	bit_set(PORTD, PD7);
}

#define dtr_on() {			\
	serjtag_busy_led_on();		\
	serjtag_close();		\
	clkout_stop();			\
	bit_clear(PORTD, PD7);		\
	timer_start();			\
}

#define dtr_off() {			\
	timer_stop();			\
	bit_set(PORTD, PD7);		\
	clkout_start();			\
	serjtag_open();			\
	serjtag_busy_led_off();		\
}

static inline void rts_on() {
	if (Baud == 1200) {
		bit_clear(DDRC, PC6);
	}
}

static inline void rts_off() {
	if (Baud == 1200) {
		bit_set(DDRC, PC6);
	}
}

#define ctrl_poll serjtag_poll
